#ifndef __KYLIN_AI_PRIVATE_OS_ASSISTANT_OS_ASSISTANT_H__
#define __KYLIN_AI_PRIVATE_OS_ASSISTANT_OS_ASSISTANT_H__

#include <functional>
#include <list>
#include <memory>
#include <vector>

#include "error.h"

namespace kyai {
namespace assistant {

// json chat result
using ChatResultCallback = std::function<void(const std::string)>;

enum PromptContentAccessType { ReadOnly = 1, ReadWrite, NoAccess };
struct Prompt {
    int id;
    std::string name;
    std::string content;
    PromptContentAccessType type;
};

struct Message {
    std::string user;
    std::string assistant;
};

class Features {
public:
    enum Feature {
        TextChat = 1,
        TextToImage = 2,
        ImageProcessing = 4,
        DocumentQA = 8
    };
    explicit Features(int features) : features_(features) {}

    bool containsFeature(Feature feature) const {
        return features_ & feature;
    }

private:
    int features_;
};

class OsAssistantPrivate;
class OsAssistant {
public:
    OsAssistant();
    ~OsAssistant();

    // blocked init
    Error init();
    Error initWithChatHistory(std::vector<Message> messages, int promptId = -1);

    void setChatAsyncCallback(ChatResultCallback callback);

    Features supportedFeatures() const;

    // json message
    void chatAsync(const std::string &message);
    void stopChat();

    void setPromptId(int promptId);

    void clearContext();

    std::vector<Prompt> prompts();
    bool setPromptOrder(std::list<int> promptIdList);

    // 创建失败返回-1
    int createPrompt(std::string name, std::string content);
    bool deletePrompt(int promptId);
    bool updatePrompt(int promptId, std::string name, std::string content);

private:
    std::unique_ptr<OsAssistantPrivate> assistantPrivate_;
};

}  // namespace assistant

}  // namespace kyai

#endif  // __KYLIN_AI_PRIVATE_OS_ASSISTANT_OS_ASSISTANT_H__
